#include "stdafx.h"
#include "scdata.h"
#include "conv.h"
#include <math.h>

CRefPoint::CRefPoint(void)
{
	m_points.SetSize(30);
	m_nPoint = m_nDrawPoint = 0;
	m_PtOrigin = -1;
	m_scale = -1;
	m_rot = 0.0;
}

CRefPoint::~CRefPoint()
{
}

void CRefPoint::DeletePoints(void)
{
	m_nPoint = 0;
}

void CRefPoint::SetLatLon(double lat, double lon)
{
	m_lat = lat;
	m_lon = lon;
}

void CRefPoint::SetLatLon(const char *lat, const char *lon)
{
	m_lat = degs2rad(lat);
	m_lon = degs2rad(lon);
}

void CRefPoint::SetScale(double scale)
{
	m_scale = scale;
}

void CRefPoint::SetLineNum(UINT linenum)
{
	m_nLineNum = linenum;
}

UINT CRefPoint::GetLineNum(void)
{
	return m_nLineNum;
}

void CRefPoint::AddPoint(double x, double y, double z)
{
	//TRACE("<Pt#%d>", m_nPoint);
	ASSERT(m_scale >  0);

	int size = m_points.GetSize();
	if (m_nPoint >= size) {
		m_points.SetSize(size + 10);
	}
	m_points[m_nPoint].m_x = x * m_scale;
	m_points[m_nPoint].m_y = y * m_scale;
	m_points[m_nPoint].m_z = z * m_scale;

	Rotate(m_points[m_nPoint].m_x, m_points[m_nPoint].m_y);

	//CalcLatLon(m_lat, m_lon, y, x, 
	//	&m_points[m_nPoint].m_lat, 
	//	&m_points[m_nPoint].m_lon);

	m_nPoint++;
}

void CRefPoint::AddMoveToPt(int n)
{
	ASSERT(m_PtOrigin >= 0);

	n -= m_PtOrigin;

	ASSERT(n < m_nPoint);

	double lat, lon;
	CalcLatLon(m_lat, m_lon, 
		m_points[n].m_y, m_points[n].m_x,
		&lat, &lon);

	AddDrawSeq(lat, lon, m_points[n].m_z, MOVETO);
}

void CRefPoint::AddDrawToPt(int n)
{
	ASSERT(m_PtOrigin >= 0);

	n -= m_PtOrigin;

	ASSERT(n < m_nPoint);

	double lat, lon;
	CalcLatLon(m_lat, m_lon, 
		m_points[n].m_y, m_points[n].m_x,
		&lat, &lon);

	AddDrawSeq(lat, lon, m_points[n].m_z, DRAWTO);
}

void CRefPoint::AddMoveTo(double x, double y, double z)
{
	double lat, lon;
	CalcLatLon(m_lat, m_lon,
		y, x,
		&lat, &lon);

	Rotate(lat, lon);
	AddDrawSeq(lat, lon, z, MOVETO);
}

void CRefPoint::AddDrawTo(double x, double y, double z)
{
	double lat, lon;
	CalcLatLon(m_lat, m_lon,
		y * m_scale, x * m_scale,
		&lat, &lon);

	Rotate(lat, lon);
	AddDrawSeq(lat, lon, z * m_scale, DRAWTO);
}

void CRefPoint::AddDrawSeq(double lat, double lon, double z, int mode)
{
	//TRACE("<DrawPt#%d>", m_nDrawPoint);

	int size = m_drawseq.GetSize();
	if (m_nDrawPoint >= size) {
		m_drawseq.SetSize(size + 10);
	}
	m_drawseq[m_nDrawPoint].m_lat = lat;
	m_drawseq[m_nDrawPoint].m_lon = lon;
	m_drawseq[m_nDrawPoint].m_z = z;
	m_drawseq[m_nDrawPoint].m_mode = mode;

	m_nDrawPoint++;
}

void CRefPoint::Draw(CDC *pDC, double latorg, double lonorg, double MeterPerPixel, BOOL checked)
{
	double lat, lon;
	int mode;
	double xx, yy;
	BOOL skip = FALSE;

	CPen pen;
	if (!checked) {
		pen.CreatePen(PS_SOLID, 0, RGB(0, 0, 0));
	} else {
		pen.CreatePen(PS_SOLID, 0, RGB(255, 0, 0));
	}

	CPen *oldPen = pDC->SelectObject(&pen);

	for (int i=0; i<m_nDrawPoint; i++) {
		lat = m_drawseq[i].m_lat;
		lon = m_drawseq[i].m_lon;
		mode = m_drawseq[i].m_mode;

		CalcOffset(latorg,  lonorg, lat, lon, &yy, &xx);
		xx = xx / MeterPerPixel;
		yy = -yy / MeterPerPixel;	// ㉺]

		if (mode == MOVETO) {
			if (xx < -3000 || xx > 3000 || yy < -3000 || yy > 3000) {
				skip = TRUE;
			} else {
				skip = FALSE;
				pDC->MoveTo( xx, yy );
			}
		}
		else {
			ASSERT(mode == DRAWTO);
			if (!skip)
				pDC->LineTo( xx, yy );
		}
	}

	pDC->SelectObject(oldPen);
}

void CRefPoint::SetPointsOrigin(int origin)
{
	ASSERT(origin >= 0);
	ASSERT(origin < 10000);	//  ad hoc ###

	m_PtOrigin = origin;
}

double CRefPoint::CalcMinDist(double lat, double lon)
{
	double mindist = 100000000.0, dist;
	double xx1, yy1,  xx2, yy2, ux, uy, dx, dy;
	double len, imul;
	BOOL firstflag = TRUE;

	for (int i=0; i<m_nDrawPoint; i++) {
		CalcOffset(lat, lon, m_drawseq[i].m_lat, m_drawseq[i].m_lon, &xx1, &yy1);

		if (!firstflag) {
			len = (xx2 - xx1) * (xx2 - xx1) + (yy2 - yy1) * (yy2 - yy1);
			len = sqrt(len);

			// PʃxNg
			ux = (xx2 - xx1) / len;
			uy = (yy2 - yy1) / len;

			// 
			imul = (-xx1) * ux + (-yy1) * uy;

			if (imul > 0 && imul < len) {
			
				// xNg
				dx = imul * ux - (-xx1);
				dy = imul * uy - (-yy1);

				// 
				dist = dx * dx + dy * dy;
				if (dist < mindist) 
					mindist = dist;
			}
		} else {
			firstflag = FALSE;
		}
		xx2 = xx1; yy2 = yy1;
	}

	return mindist;	// Q拗Ԃ
}



void CRefPoint::SetRotate(double rot)
{
	m_rot = rot / 180.0 * PI;
	m_cos = cos(m_rot);
	m_sin = sin(m_rot);
}

void CRefPoint::Rotate(double &x, double &y)
{
	if (m_rot == 0.0) return;

	double xx = x * m_cos + y * m_sin;
	double yy = -x * m_sin + y * m_cos;
	x = xx; y = yy;
}
